@@ -13,20 +13,16 @@ use Queue::DBI::Element;
 
 =head1 NAME
 
-Queue::DBI - A queueing module with an emphasis on safety, using DBI as a
-storage system for queued data.
+Queue::DBI - A queueing module with an emphasis on safety, using DBI as a storage system for queued data.
 
 
 =head1 VERSION
 
-Version 1.6
+Version 2.0.0
 
 =cut
 
-our $VERSION = '1.6';
-
-
-our $UNLIMITED_RETRIES = -1;
+our $VERSION = '2.0.0';
 
 our $DEFAULT_QUEUES_TABLE_NAME = 'queues';
 
@@ -114,7 +110,7 @@ Mandatory, the name of the queue elements will be added to / removed from.
 
 =item * 'database handle'
 
-Mandatory, a DBI / DBD::Mysql object.
+Mandatory, a DBI object.
 
 =item * 'cleanup_timeout'
 
@@ -124,7 +120,9 @@ that time.
 
 =item * 'verbose'
 
-Optional, see verbose() for options.
+Optional, control the verbosity of the warnings in the code. 0 will not display
+any warning; 1 will only give one line warnings about the current operation;
+2 will also usually output the SQL queries performed.
 
 =item * 'max_requeue_count'
 
@@ -145,6 +143,12 @@ By default, Queue::DBI uses a table named 'queue_elements' to store the queued
 data. This allows using your own name, if you want to support separate queuing
 systems or legacy systems.
 
+=item * 'lifetime'
+
+By default, Queue:::DBI will fetch elements regardless of how old they are. Use
+this option to specify how old (in seconds) an element can be and still be
+retrieved for processing.
+
 =back
 
 =cut
@@ -156,11 +160,13 @@ sub new
 	# Check parameters.
 	foreach my $arg ( qw( queue_name database_handle ) )
 	{
-		die "Argument '$_' is needed to create the Queue::DBI object"
-			unless defined( $args{$arg} ) && ( $args{$arg} ne '' );
+		croak "Argument '$arg' is needed to create the Queue::DBI object"
+			if !defined( $args{$arg} ) || ( $args{$arg} eq '' );
 	}
-	die 'Cleanup timeout must be an integer representing seconds'
+	croak 'Cleanup timeout must be an integer representing seconds'
 		if defined( $args{'cleanup_timeout'} ) && ( $args{'cleanup_timeout'} !~ m/^\d+$/ );
+	croak 'Lifetime must be an integer representing seconds'
+		if defined( $args{'lifetime'} ) && ( $args{'lifetime'} !~ m/^\d+$/ );
 	
 	# Create the object.
 	my $dbh = $args{'database_handle'};
@@ -173,7 +179,6 @@ sub new
 				'queues'         => $args{'queues_table_name'},
 				'queue_elements' => $args{'queue_elements_table_name'},
 			},
-			'verbose'     => 0,
 		},
 		$class
 	);
@@ -191,19 +196,16 @@ sub new
 		{},
 		$args{'queue_name'},
 	);
-	die "The queue >$args{'queue_name'}< doesn't exist in the lookup table."
+	croak "The queue >$args{'queue_name'}< doesn't exist in the lookup table."
 		unless defined( $queue[0] ) && ( $queue[0] =~ m/^\d+$/ );
-	$self->{'queue_id'} = $queue[0],
+	$self->{'queue_id'} = $queue[0];
 	
-	$self->verbose( $args{'verbose'} )
-		if defined( $args{'verbose'} );
-	
-	$self->max_requeue_count(
-		defined( $args{'max_requeue_count'} )
-			? $args{'max_requeue_count'}
-			: $Queue::DBI::UNLIMITED_RETRIES
-	);
+	# Set optional parameters.
+	$self->set_verbose( $args{'verbose'} );
+	$self->set_max_requeue_count( $args{'max_requeue_count'} );
+	$self->set_lifetime( $args{'lifetime'} );
 	
+	# Perform queue cleanup if a timeout is specified.
 	$self->cleanup( $args{'cleanup_timeout'} )
 		if defined( $args{'cleanup_timeout'} );
 	
@@ -211,74 +213,6 @@ sub new
 }
 
 
-=head2 verbose()
-
-Control the verbosity of the warnings in the code.
-
-	$queue->verbose(1); # turn on verbose information
-	
-	$queue->verbose(2); # be extra verbose
-	
-	$queue->verbose(0); # quiet now!
-	
-	warn 'Verbose' if $queue->verbose(); # getter-style
-	
-	warn 'Very verbose' if $queue->verbose() > 1;
-	
-0 will not display any warning, 1 will only give one line warnings about the
-current operation and 2 will also usually output the SQL queries performed.
-
-=cut
-
-sub verbose
-{
-	my ( $self, $verbose ) = @_;
-	
-	$self->{'verbose'} = ( $verbose || 0 )
-		if defined( $verbose );
-	
-	return $self->{'verbose'};
-}
-
-
-=head2 max_requeue_count()
-
-Sets the number of time an element can be requeued before it is ignored when
-retrieving elements. Set it to $Queue::DBI::UNLIMITED_RETRIES to reset
-Queue::DBI back to its default behavior of re-pulling elements without limit.
-
-	# Don't keep pulling the element if it has been requeued more than 5 times.
-	$queue->max_requeue_count( 5 );
-	
-	# Keep pulling elements regardless of the number of times they have been
-	# requeued.
-	$queue->max_requeue_count( $UNLIMITED_RETRIES );
-	
-	# Find how many times a queue object will try to requeue.
-	my $max_requeue_count = $queue->max_requeue_count();
-
-=cut
-
-sub max_requeue_count
-{
-	my ( $self, $max_requeue_count ) = @_;
-	
-	if ( defined( $max_requeue_count ) )
-	{
-		if ( ( $max_requeue_count =~ m/^\d+$/ ) || ( $max_requeue_count eq $UNLIMITED_RETRIES ) )
-		{
-			$self->{'max_requeue_count'} = $max_requeue_count;
-		}
-		else
-		{
-			die 'max_requeue_count must be an integer or $Queue::DBI::UNLIMITED_RETRIES';
-		}
-	}
-	
-	return $self->{'max_requeue_count'};
-}
-
-
 =head2 get_queue_id()
 
 Returns the queue ID corresponding to the current queue object.
@@ -304,7 +238,7 @@ Returns the number of elements in the queue.
 sub count
 {
 	my ( $self ) = @_;
-	my $verbose = $self->verbose();
+	my $verbose = $self->get_verbose();
 	my $dbh = $self->get_dbh();
 	carp "Entering count()." if $verbose;
 	
@@ -319,7 +253,7 @@ sub count
 		),
 		{},
 		$self->get_queue_id(),
-	) || die 'Cannot execute SQL: ' . $dbh->errstr;
+	) || croak 'Cannot execute SQL: ' . $dbh->errstr();
 	
 	my $element_count = defined( $data ) && defined( $data->[0] ) ? $data->[0] : 0;
 	
@@ -344,13 +278,13 @@ as it is serialized for storage in the database.
 sub enqueue
 {
 	my ( $self, $data ) = @_;
-	my $verbose = $self->verbose();
+	my $verbose = $self->get_verbose();
 	my $dbh = $self->get_dbh();
 	carp "Entering enqueue()." if $verbose;
 	carp "Data is: " . Dumper( $data ) if $verbose > 1;
 	
 	my $encoded_data = MIME::Base64::encode_base64( Storable::freeze( $data ) );
-	die 'The size of the data to store exceeds the maximum internal storage size available.'
+	croak 'The size of the data to store exceeds the maximum internal storage size available.'
 		if length( $encoded_data ) > $MAX_VALUE_SIZE;
 	
 	$dbh->do(
@@ -365,7 +299,7 @@ sub enqueue
 		$self->get_queue_id(),
 		$encoded_data,
 		time(),
-	) || die 'Cannot execute SQL: ' . $dbh->errstr();
+	) || croak 'Cannot execute SQL: ' . $dbh->errstr();
 	
 	# We need to reset the internal cached value preventing infinite loops, other-
 	# wise this new element will not be taken into account by the current queue
@@ -374,7 +308,12 @@ sub enqueue
 	
 	carp "Element inserted, leaving enqueue()." if $verbose;
 	
-	return $dbh->{'mysql_insertid'};
+	return $dbh->last_insert_id(
+		undef,
+		undef,
+		$self->get_queue_elements_table_name(),
+		'queue_element_id',
+	);
 }
 
 
@@ -397,10 +336,10 @@ elements can be specified using 'search_in_ids':
 
 =cut
 
-sub next
+sub next ## no critic (Subroutines::ProhibitBuiltinHomonyms)
 {
 	my ( $self, %args ) = @_;
-	my $verbose = $self->verbose();
+	my $verbose = $self->get_verbose();
 	carp "Entering next()." if $verbose;
 	
 	my $elements = $self->retrieve_batch(
@@ -446,12 +385,12 @@ elements can be specified using 'search_in_ids':
 sub retrieve_batch
 {
 	my ( $self, $number_of_elements_to_retrieve, %args ) = @_;
-	my $verbose = $self->verbose();
+	my $verbose = $self->get_verbose();
 	my $dbh = $self->get_dbh();
 	carp "Entering retrieve_batch()." if $verbose;
 	
 	# Check parameters
-	die 'The number of elements to retrieve from the queue is not properly formatted'
+	croak 'The number of elements to retrieve from the queue is not properly formatted'
 		unless defined( $number_of_elements_to_retrieve ) && ( $number_of_elements_to_retrieve =~ m/^\d+$/ );
 	
 	# Prevent infinite loops
@@ -469,7 +408,7 @@ sub retrieve_batch
 			{},
 			$self->get_queue_id(),
 		);
-		die 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
+		croak 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
 		if ( defined( $data ) && defined( $data->[0] ) )
 		{
 			$self->{'max_id'} = $data->[0];
@@ -478,7 +417,7 @@ sub retrieve_batch
 		{
 			# Empty queue
 			carp "Detected empty queue, leaving." if $verbose;
-			return undef;
+			return;
 		}
 	}
 	
@@ -494,21 +433,29 @@ sub retrieve_batch
 	}
 	
 	# Make sure we don't use requeued elements more times than specified.
-	my $max_requeue_count = $self->max_requeue_count();
-	my $sql_max_requeue_count = defined( $max_requeue_count ) && ( $max_requeue_count != $UNLIMITED_RETRIES )
+	my $max_requeue_count = $self->get_max_requeue_count();
+	my $sql_max_requeue_count = defined( $max_requeue_count )
 		? 'AND requeue_count <= ' . $dbh->quote( $max_requeue_count )
 		: '';
 	
-	# Retrieve the first available elements from the queue
-	carp "Retrieving data." if $verbose;
-	carp "Parameters:\n\tLast ID: $self->{'last_id'}\n\tMax ID: $self->{'max_id'}\n" if $verbose > 1;
+	# Make sure we don't use elements that exceed the specified lifetime.
+	my $lifetime = $self->get_lifetime();
+	my $sql_lifetime = defined( $lifetime )
+		? 'AND created >= ' . ( time() - $lifetime )
+		: '';
+	
+	# If specified, retrieve only those IDs.
 	my $ids = defined( $args{'search_in_ids'} )
 		? 'AND queue_element_id IN (' . join( ',', map { $dbh->quote( $_ ) } @{ $args{'search_in_ids' } } ) . ')'
 		: '';
+	
+	# Retrieve the first available elements from the queue.
+	carp "Retrieving data." if $verbose;
+	carp "Parameters:\n\tLast ID: $self->{'last_id'}\n\tMax ID: $self->{'max_id'}\n" if $verbose > 1;
 	my $data = $dbh->selectall_arrayref(
 		sprintf(
 			q|
-				SELECT queue_element_id, data, requeue_count
+				SELECT queue_element_id, data, requeue_count, created
 				FROM %s
 				WHERE queue_id = ?
 					AND lock_time IS NULL
@@ -516,12 +463,14 @@ sub retrieve_batch
 					AND queue_element_id <= ?
 					%s
 					%s
+					%s
 				ORDER BY queue_element_id ASC
 				LIMIT ?
 			|,
 			$dbh->quote_identifier( $self->get_queue_elements_table_name() ),
 			$ids,
 			$sql_max_requeue_count,
+			$sql_lifetime,
 		),
 		{},
 		$self->get_queue_id(),
@@ -529,11 +478,11 @@ sub retrieve_batch
 		$self->{'max_id'},
 		$number_of_elements_to_retrieve,
 	);
-	die 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
+	croak 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
 	
 	# All the remaining elements are locked
 	return []
-		unless defined( $data ) && ( scalar( @$data) != 0 );
+		if !defined( $data ) || ( scalar( @$data) == 0 );
 	
 	# Create objects
 	carp "Creating new Queue::DBI::Element objects." if $verbose;
@@ -547,6 +496,7 @@ sub retrieve_batch
 				'data'          => Storable::thaw( MIME::Base64::decode_base64( $row->[1] ) ),
 				'id'            => $row->[0],
 				'requeue_count' => $row->[2],
+				'created'       => $row->[3],
 			)
 		);
 	}
@@ -576,12 +526,12 @@ This method requires a queue element ID to be passed as parameter.
 sub get_element_by_id
 {
 	my ( $self, $queue_element_id ) = @_;
-	my $verbose = $self->verbose();
+	my $verbose = $self->get_verbose();
 	my $dbh = $self->get_dbh();
 	carp "Entering get_element_by_id()." if $verbose;
 	
 	# Check parameters.
-	die 'A queue element ID is required by this method'
+	croak 'A queue element ID is required by this method'
 		unless defined( $queue_element_id );
 	
 	# Retrieve the specified element from the queue.
@@ -600,11 +550,10 @@ sub get_element_by_id
 		$self->get_queue_id(),
 		$queue_element_id,
 	);
-	die 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
+	croak 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
 	
 	# Queue element ID doesn't exist or belongs to another queue.
-	return undef
-		unless defined( $data );
+	return unless defined( $data );
 	
 	# Create the Queue::DBI::Element object.
 	carp "Creating a new Queue::DBI::Element object." if $verbose;
@@ -614,6 +563,7 @@ sub get_element_by_id
 		'data'          => Storable::thaw( MIME::Base64::decode_base64( $data->{'data'} ) ),
 		'id'            => $data->{'queue_element_id'},
 		'requeue_count' => $data->{'requeue_count'},
+		'created'       => $data->{'created'},
 	);
 	
 	carp "Leaving get_element_by_id()." if $verbose;
@@ -639,12 +589,12 @@ Returns the items requeued so that a specific action can be taken on them.
 sub cleanup
 {
 	my ( $self, $time_in_seconds ) = @_;
-	my $verbose = $self->verbose();
+	my $verbose = $self->get_verbose();
 	my $dbh = $self->get_dbh();
 	carp "Entering cleanup()." if $verbose;
 	
 	$time_in_seconds ||= '';
-	die 'Time in seconds is not correctly formatted'
+	croak 'Time in seconds is not correctly formatted'
 		unless $time_in_seconds =~ m/^\d+$/;
 	
 	# Find all the orphans
@@ -663,7 +613,7 @@ sub cleanup
 		$self->get_queue_id(),
 		time() - $time_in_seconds,
 	);
-	die 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
+	croak 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
 	return []
 		unless defined( $rows );
 	
@@ -691,94 +641,289 @@ sub cleanup
 }
 
 
+=head2 purge()
+
+Remove (permanently, caveat emptor!) queue elements based on how many times
+they've been requeued or how old they are.
+
+	# Remove permanently elements that have been requeued more than 10 times.
+	$queue->purge( max_requeue_count => 10 );
+	
+	# Remove permanently elements that were created over an hour ago.
+	$queue->purge( lifetime => 3600 );
+
+Important: locked elements are not purged even if they match the criteria, as
+they are presumed to be currently in process and purging them would create
+unexpected failures in the application processing them.
+
+Also note that I<max_requeue_count> and I<lifetime> cannot be combined.
+
+=cut
+
+sub purge
+{
+	my ( $self, %args ) = @_;
+	my $verbose = $self->get_verbose();
+	my $dbh = $self->get_dbh();
+	carp "Entering cleanup()." if $verbose;
+	
+	my $max_requeue_count = $args{'max_requeue_count'};
+	my $lifetime = $args{'lifetime'};
+	
+	# Check parameters.
+	croak 'Cleanup timeout must be an integer representing seconds'
+		if defined( $max_requeue_count ) && ( $max_requeue_count !~ m/^\d+$/ );
+	croak 'Lifetime must be an integer representing seconds'
+		if defined( $lifetime ) && ( $lifetime !~ m/^\d+$/ );
+	croak '"max_requeue_count" and "lifetime" cannot be combined, specify one OR the other'
+		if defined( $lifetime ) && defined( $max_requeue_count );
+	croak '"max_requeue_count" or "lifetime" must be specified'
+		if !defined( $lifetime ) && !defined( $max_requeue_count );
+	
+	# Prepare query clauses.
+	my $sql_lifetime = defined( $lifetime )
+		? 'AND created < ' . ( time() - $lifetime )
+		: '';
+	my $sql_max_requeue_count = defined( $max_requeue_count )
+		? 'AND requeue_count > ' . $dbh->quote( $max_requeue_count )
+		: '';
+	
+	# Purge the queue.
+	my $rows_deleted = $dbh->do(
+		sprintf(
+			q|
+				DELETE
+				FROM %s
+				WHERE queue_id = ?
+					AND lock_time IS NULL
+					%s
+					%s
+			|,
+			$dbh->quote_identifier( $self->get_queue_elements_table_name() ),
+			$sql_lifetime,
+			$sql_max_requeue_count,
+		),
+		{},
+		$self->get_queue_id(),
+	);
+	croak 'Cannot execute SQL: ' . $dbh->errstr() if defined( $dbh->errstr() );
+	
+	carp "Leaving cleanup()." if $verbose;
+	# Account for '0E0' which means no rows affected, and translates into no
+	# rows deleted in our case.
+	return $rows_deleted eq '0E0'
+		? 0
+		: $rows_deleted;
+}
+
+
+=head1 ACCESSORS
+
+=head2 get_max_requeue_count()
+
+Return how many times an element can be requeued before it is ignored when
+retrieving elements.
+
+	my $max_requeue_count = $queue->get_max_requeue_count();
+
+=cut
+
+sub get_max_requeue_count
+{
+	my ( $self ) = @_;
+	
+	return $self->{'max_requeue_count'};
+}
+
+
+=head2 set_max_requeue_count()
+
+Set the number of time an element can be requeued before it is ignored when
+retrieving elements. Set it to C<undef> to disable the limit.
+
+	# Don't keep pulling the element if it has been requeued more than 5 times.
+	$queue->set_max_requeue_count( 5 );+
+	
+	# Retry without limit.
+	$queue->set_max_requeue_count( undef );
+
+=cut
+
+sub set_max_requeue_count
+{
+	my ( $self, $max_requeue_count ) = @_;
+	
+	croak 'max_requeue_count must be an integer or undef'
+		if defined( $max_requeue_count ) && ( $max_requeue_count !~ /^\d+$/ );
+	
+	$self->{'max_requeue_count'} = $max_requeue_count;
+	
+	return;
+}
+
+
+=head2 get_lifetime()
+
+Return how old an element can be before it is ignored when retrieving elements.
+
+	# Find how old an element can be before the queue will stop retrieving it.
+	my $lifetime = $queue->get_lifetime();
+
+=cut
+
+sub get_lifetime
+{
+	my ( $self ) = @_;
+	
+	return $self->{'lifetime'};
+}
+
+
+=head2 set_lifetime()
+
+Set how old an element can be before it is ignored when retrieving elements.
+
+Set it to C<undef> to reset Queue::DBI back to its default behavior of
+retrieving elements without time limit.
+
+	# Don't pull queue elements that are more than an hour old.
+	$queue->set_lifetime( 3600 );
+	
+	# Pull elements without time limit.
+	$queue->set_lifetime( undef );
+
+=cut
+
+sub set_lifetime
+{
+	my ( $self, $lifetime ) = @_;
+	
+	croak 'lifetime must be an integer or undef'
+		if defined( $lifetime ) && ( $lifetime !~ /^\d+$/ );
+	
+	$self->{'lifetime'} = $lifetime;
+	
+	return;
+}
+
+
+=head2 get_verbose()
+
+Return the verbosity level, which is used in the module to determine when and
+what type of debugging statements / information should be warned out.
+
+See C<set_verbose()> for the possible values this function can return.
+
+	warn 'Verbose' if $queue->get_verbose();
+	
+	warn 'Very verbose' if $queue->get_verbose() > 1;
+
+=cut
+
+sub get_verbose
+{
+	my ( $self ) = @_;
+	
+	return $self->{'verbose'};
+}
+
+
+=head2 set_verbose()
+
+Control the verbosity of the warnings in the code:
+
+=over 4
+
+=item * 0 will not display any warning;
+
+=item * 1 will only give one line warnings about the current operation;
+
+=item * 2 will also usually output the SQL queries performed.
+
+=back
+
+	$queue->set_verbose(1); # turn on verbose information
+	
+	$queue->set_verbose(2); # be extra verbose
+	
+	$queue->set_verbose(0); # quiet now!
+
+=cut
+
+sub set_verbose
+{
+	my ( $self, $verbose ) = @_;
+	
+	$self->{'verbose'} = ( $verbose || 0 );
+	
+	return;
+}
+
+
+=head1 DEPRECATED METHODS
+
 =head2 create_tables()
 
-Creates the tables in the database the database handle passed as parameter
-points to. This allows setting up Queue::DBI's underlying database structure
-quickly.
+Please use C<create_tables()> in C<Queue::DBI::Admin> instead.
 
-	Queue::DBI::create_tables(
-		dbh           => $dbh,
+Here is an example that shows how to refactor your call to this deprecated
+function:
+
+	# Load the admin module.
+	use Queue::DBI::Admin;
+	
+	# Create the object which will allow managing the queues.
+	my $queues_admin = Queue::DBI::Admin->new(
+		database_handle => $dbh,
+	);
+	
+	# Create the tables required by Queue::DBI to store the queues and data.
+	$queues_admin->create_tables(
 		drop_if_exist => $boolean,
 		sqlite        => $boolean,
 	);
 
-By default, it won't drop any table but you can force that by setting
-'drop_if_exist' to 1. 'sqlite' is also set to 0 by default, as this parameter
-is used only for testing.
-
 =cut
 
 sub create_tables
 {
-	my ( %args ) = @_;
-	
-	# Check arguments.
-	die 'Missing database handle'
-		unless defined( $args{'dbh'} );
-	
-	foreach my $arg ( qw( drop_if_exist sqlite ) )
-	{
-		$args{$arg} = 0
-			unless defined( $args{$arg} ) && $args{$arg};
-	}
-	
-	# Create the list of queues.
-	$args{'dbh'}->do( q|DROP TABLE IF EXISTS `queues`| )
-		if $args{'drop_if_exist'};
-	$args{'dbh'}->do(
-		$args{'sqlite'}
-		? q|
-			CREATE TABLE `queues`
-			(
-				`queue_id` INTEGER PRIMARY KEY AUTOINCREMENT,
-				`name` VARCHAR(255) NOT NULL UNIQUE
-			)
-		|
-		: q|
-			CREATE TABLE `queues`
-			(
-				`queue_id` INT(11) NOT NULL AUTO_INCREMENT,
-				`name` VARCHAR(255) NOT NULL,
-				PRIMARY KEY (`queue_id`),
-				UNIQUE KEY `name` (`name`)
-			)
-			ENGINE=InnoDB
-		|
-	);
-	
-	# Create the table that will hold the queue elements.
-	$args{'dbh'}->do( q|DROP TABLE IF EXISTS `queue_elements`| )
-		if $args{'drop_if_exist'};
-	$args{'dbh'}->do(
-		$args{'sqlite'}
-		? q|
-			CREATE TABLE `queue_elements`
-			(
-				`queue_element_id` INTEGER PRIMARY KEY AUTOINCREMENT,
-				`queue_id` INTEGER NOT NULL,
-				`data` TEXT,
-				`lock_time` INT(10) DEFAULT NULL,
-				`requeue_count` INT(3) DEFAULT '0',
-				`created` INT(10) NOT NULL DEFAULT '0'
-			)
-		|
-		: q|
-			CREATE TABLE `queue_elements`
-			(
-				`queue_element_id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
-				`queue_id` INT(11) NOT NULL,
-				`data` TEXT,
-				`lock_time` INT(10) UNSIGNED DEFAULT NULL,
-				`requeue_count` INT(3) UNSIGNED DEFAULT '0',
-				`created` INT(10) UNSIGNED NOT NULL DEFAULT '0',
-				PRIMARY KEY (`queue_element_id`),
-				KEY `idx_fk_queue_id` (`queue_id`),
-				CONSTRAINT `queue_element_ibfk_1` FOREIGN KEY (`queue_id`) REFERENCES `queue` (`queue_id`)
-			)
-			ENGINE=InnoDB
-		|
-	);
+	croak 'create_tables() in Queue::DBI has been deprecated, please use create_tables() in Queue::DBI::Admin instead.';
+}
+
+
+=head2 lifetime()
+
+Please use C<get_lifetime()> and C<set_lifetime()> instead.
+
+=cut
+
+sub lifetime
+{
+	croak 'lifetime() has been deprecated, please use get_lifetime() / set_lifetime() instead.';
+}
+
+
+=head2 verbose()
+
+Please use C<get_verbose()> and C<set_verbose()> instead.
+
+=cut
+
+sub verbose
+{
+	croak 'verbose() has been deprecated, please use get_verbose() / set_verbose() instead.';
+}
+
+
+=head2 max_requeue_count()
+
+Please use C<get_max_requeue_count()> and C<set_max_requeue_count()> instead.
+
+=cut
+
+sub max_requeue_count
+{
+	croak 'max_requeue_count() has been deprecated, please use get_max_requeue_count() / set_max_requeue_count() instead.';
 }
 
 
@@ -843,7 +988,7 @@ Guillaume Aubert, C<< <aubertg at cpan.org> >>.
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-queue-safe at rt.cpan.org>, or through
+Please report any bugs or feature requests to C<bug-queue-dbi at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Queue-DBI>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
@@ -880,14 +1025,23 @@ L<http://search.cpan.org/dist/Queue-DBI/>
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to Geeknet, Inc. L<http://www.geek.net> for funding the initial development of this code!
+Thanks to ThinkGeek (L<http://www.thinkgeek.com/>) and its corporate overlords
+at Geeknet (L<http://www.geek.net/>), for footing the bill while I write code
+for them!
 
-Thanks to Jamie McCarthy for the locking mechanism improvements in version 1.1.
+Thanks to Jacob Rose C<< <jacob at thinkgeek.com> >>, who wrote the first
+queueing module at ThinkGeek L<http://www.thinkgeek.com> and whose work
+provided the inspiration to write this full-fledged queueing system.
+
+Thanks to Jamie McCarthy for the locking mechanism improvements in version 1.1.0.
+
+Thanks to Sergey Bond for suggesting many features added in version 1.8.x
+(lifetime constraint, purge() function, get/set functions cleanup).
 
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 Guillaume Aubert.
+Copyright 2009-2012 Guillaume Aubert.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Artistic License.
