@@ -6,17 +6,19 @@ use strict;
 use Data::Dumper;
 use Carp;
 
+
 =head1 NAME
 
-Queue::DBI::Element - An object representing an element pulled from the queue
+Queue::DBI::Element - An object representing an element pulled from the queue.
+
 
 =head1 VERSION
 
-Version 1.6
+Version 2.0.0
 
 =cut
 
-our $VERSION = '1.6';
+our $VERSION = '2.0.0';
 
 
 =head1 SYNOPSIS
@@ -34,6 +36,7 @@ Create a new Queue::DBI::Element object.
 		'data'          => $data,
 		'id'            => $id,
 		'requeue_count' => $requeue_count,
+		'created'       => $created,
 	);
 
 All parameters are mandatory and correspond respectively to the Queue::DBI
@@ -52,12 +55,12 @@ sub new
 	my ( $class, %args ) = @_;
 	
 	# Check parameters
-	foreach my $arg ( qw( data id requeue_count ) )
+	foreach my $arg ( qw( data id requeue_count created ) )
 	{
-		die "Argument '$arg' is needed to create the Queue::DBI object"
-			unless defined( $args{$arg} ) && ( $args{$arg} ne '' );
+		croak "Argument '$arg' is needed to create the Queue::DBI object"
+			if !defined( $args{$arg} ) || ( $args{$arg} eq '' );
 	}
-	die 'Pass a Queue::DBI object to create an Queue::DBI::Element object'
+	croak 'Pass a Queue::DBI object to create an Queue::DBI::Element object'
 		unless defined( $args{'queue'} ) && $args{'queue'}->isa( 'Queue::DBI' );
 	
 	# Create the object
@@ -67,6 +70,7 @@ sub new
 			'data'          => $args{'data'},
 			'id'            => $args{'id'},
 			'requeue_count' => $args{'requeue_count'},
+			'created'       => $args{'created'},
 		},
 		$class
 	);
@@ -91,11 +95,11 @@ of it
 
 =cut
 
-sub lock
+sub lock ## no critic (Subroutines::ProhibitBuiltinHomonyms)
 {
 	my ( $self ) = @_;
-	my $queue = $self->queue();
-	my $verbose = $queue->verbose();
+	my $queue = $self->get_queue();
+	my $verbose = $queue->get_verbose();
 	my $dbh = $queue->get_dbh();
 	carp "Entering lock()." if $verbose;
 	
@@ -112,7 +116,7 @@ sub lock
 		{},
 		time(),
 		$self->id(),
-	) || die 'Cannot lock element: ' . $dbh->errstr;
+	) || croak 'Cannot lock element: ' . $dbh->errstr;
 	
 	my $success = ( defined( $rows ) && ( $rows == 1 ) ) ? 1 : 0;
 	carp "Element locked: " . ( $success ? 'success' : 'already locked or gone' ) . "." if $verbose;
@@ -140,8 +144,8 @@ In case the processing of an element has failed
 sub requeue
 {
 	my ( $self ) = @_;
-	my $queue = $self->queue();
-	my $verbose = $queue->verbose();
+	my $queue = $self->get_queue();
+	my $verbose = $queue->get_verbose();
 	my $dbh = $queue->get_dbh();
 	carp "Entering requeue()." if $verbose;
 	
@@ -209,8 +213,8 @@ completed.
 sub success
 {
 	my ( $self ) = @_;
-	my $queue = $self->queue();
-	my $verbose = $queue->verbose();
+	my $queue = $self->get_queue();
+	my $verbose = $queue->get_verbose();
 	my $dbh = $queue->get_dbh();
 	carp "Entering success()." if $verbose;
 	
@@ -235,7 +239,7 @@ sub success
 	
 	if ( ! defined( $rows ) || $rows == -1 )
 	{
-		die 'Cannot remove element: ' . $dbh->errstr();
+		croak 'Cannot remove element: ' . $dbh->errstr();
 	}
 	
 	my $success = 0;
@@ -249,25 +253,25 @@ sub success
 	{
 		# No LOCKED element found to delete, try to find an UNLOCKED one in case it
 		# got requeued by a parallel process.
-		my $rows = $dbh->do(
+		my $deleted_rows = $dbh->do(
 			sprintf(
 				q|
 					DELETE
 					FROM %s
 					WHERE queue_element_id = ?
 				|,
-				$dbh->quote_identifier( $self->get_queues_table_name() ),
+				$dbh->quote_identifier( $queue->get_queue_elements_table_name() ),
 			),
 			{},
 			$self->id(),
 		);
 		
-		if ( ! defined( $rows ) || $rows == -1 )
+		if ( ! defined( $deleted_rows ) || $deleted_rows == -1 )
 		{
-			die 'Cannot remove element: ' . $dbh->errstr;
+			croak 'Cannot remove element: ' . $dbh->errstr;
 		}
 		
-		if ( $rows == 1 )
+		if ( $deleted_rows == 1 )
 		{
 			# An UNLOCKED element was found and deleted. It probably means that
 			# another process is still working on that element as well (possibly
@@ -316,6 +320,8 @@ sub data
 
 Returns the number of times that the current element has been requeued.
 
+	my $requeue_count = $element->requeue_count();
+
 =cut
 
 sub requeue_count
@@ -325,6 +331,7 @@ sub requeue_count
 	return $self->{'requeue_count'};
 }
 
+
 =head2 id()
 
 Returns the ID of the current element
@@ -341,15 +348,59 @@ sub id
 }
 
 
-=head1 INTERNAL METHODS
+=head2 get_created_time()
 
-=head2 queue()
+Returns the unixtime at which the element was originally created.
 
-Returns the Queue::DBI object used to pull the current element.
+	my $created = $element->get_created_time();
 
 =cut
 
-sub queue
+sub get_created_time
+{
+	my ( $self ) = @_;
+	
+	return $self->{'created'};
+}
+
+
+=head2 is_over_lifetime()
+
+Returns a boolean indicating whether the current element is over the lifetime
+specified when instanciating the queue. This is especially helpful if you
+retrieve a large batch of elements and do long processing operations on each
+of them.
+
+	my $is_over_lifetime = $element->is_over_lifetime();
+
+=cut
+
+sub is_over_lifetime
+{
+	my ( $self ) = @_;
+	my $queue = $self->get_queue();
+	my $lifetime = $queue->get_lifetime();
+	
+	# If the queue doesn't a lifetime, an element will never "expire".
+	return 0 if !defined( $lifetime );
+	
+	# Check the time the element was created.
+	my $created_time = $self->get_created_time();
+	return time() - $created_time > $lifetime;
+}
+
+
+=head1 INTERNAL METHODS
+
+=head2 get_queue()
+
+Returns the Queue::DBI object used to pull the current element.
+
+	my $queue = $element->get_queue();
+
+=cut
+
+sub get_queue
 {
 	my ( $self ) = @_;
 	
@@ -364,7 +415,7 @@ Guillaume Aubert, C<< <aubertg at cpan.org> >>.
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-queue-safe at rt.cpan.org>, or through
+Please report any bugs or feature requests to C<bug-queue-dbi at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Queue-DBI>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
@@ -373,7 +424,7 @@ automatically be notified of progress on your bug as I make changes.
 
 You can find documentation for this module with the perldoc command.
 
-	perldoc Queue::DBI
+	perldoc Queue::DBI::Element
 
 
 You can also look for information at:
@@ -401,12 +452,23 @@ L<http://search.cpan.org/dist/Queue-DBI/>
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to Geeknet, Inc. L<http://www.geek.net> for funding the initial development of this code!
+Thanks to ThinkGeek (L<http://www.thinkgeek.com/>) and its corporate overlords
+at Geeknet (L<http://www.geek.net/>), for footing the bill while I write code
+for them!
+
+Thanks to Jacob Rose C<< <jacob at thinkgeek.com> >>, who wrote the first
+queueing module at ThinkGeek L<http://www.thinkgeek.com> and whose work
+provided the inspiration to write this full-fledged queueing system.
+
+Thanks to Jamie McCarthy for the locking mechanism improvements in version 1.1.0.
+
+Thanks to Sergey Bond for suggesting many features added in version 1.8.x
+(lifetime constraint, purge() function, get/set functions cleanup).
 
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 Guillaume Aubert.
+Copyright 2009-2012 Guillaume Aubert.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Artistic License.
